
Capistrano::Configuration.instance.load do
  
  require 'capistrano'
  require 'ratchetify/helpers'
  require 'ratchetify/setup'
  require 'ratchetify/domain'
  require 'ratchetify/service'
  require 'ratchetify/db'
  require 'ratchetify/git'
  
  before "create:rails", "setup:environment"
  before "create:rails", "git:clone_repo"
  before "create:rails", "db:create"
  before "create:rails", "db:create_database_yml"
  after "create:rails", "service:create"
  after "create:rails", "service:create_reverse_proxy"
  after "create:rails", "domain:add"
  
  desc "Deploy an app for the first time"
  namespace :create do
    
    desc "Deploy a RAILS app to uberspace"
    task :rails do
      config_rails_app
      finalize
    end
    
    task :config_rails_app do
      # run bundle install first
      run "cd #{deploy_dir} && bundle install --path ~/.gem"

      # application.yml should normally be on the .gitignore list, however you application might need 
      # a basic configuration file to e.g. load seed data. Provide such a basic file as 'application.example.yml'
      # and it will be used to configure the app initially.
      
      # create a default application.yml file if non was provided
      if not file_exists? "#{deploy_dir}/config/application.yml"
        if file_exists? "#{deploy_dir}/config/application.example.yml"
          run "cd #{deploy_dir}/config && cp application.example.yml application.yml"
        end
      end
      
      # run rake db:migrate to create all tables etc
      run "cd #{deploy_dir} && bundle exec rake db:migrate RAILS_ENV=#{fetch :environment}"
      
      # run rake db:seed to load seed data
      run "cd #{deploy_dir} && bundle exec rake db:seed RAILS_ENV=#{fetch :environment}"
      
      # compile assets from the asset pipeline
      run "cd #{deploy_dir} && bundle exec rake assets:precompile RAILS_ENV=#{fetch :environment}"
      
      # create symbolic links so that apache will find the assets
      run "cd #{deploy_dir} && ln -s public/assets assets"
    end
    
    task :finalize do
      # create symbolic links..
      run "cd #{webroot_dir} && ln -s #{deploy_dir} #{fetch :host}.#{fetch :domain}"
      run "cd #{webroot_dir} && ln -s #{deploy_dir} #{fetch :domain}" unless (wildcard_domain == false)
    end
    
  end # namespace
end
